/***************************************************************************

 Author        : Neil Zhao - CAST

 Date          : Feb 20th, 2009

 File          : AD7190_IO.c

 Hardware      : ADuC7026 and AD7190

 Description   : Use the GPIO to simulate the SPI communication of AD7190
	
***************************************************************************/
#include <ADuC7026.h>
#include "AD7190_IO.h"

#define SET_CS()		GP4DAT = (GP4DAT | 0x00080000)	//P4.3->/CS
#define CLR_CS()		GP4DAT = (GP4DAT & 0xFFF7FFFF)

#define	SET_SCL()		GP4DAT = (GP4DAT | 0x00100000)	//P4.4->SCLK
#define	CLR_SCL()		GP4DAT = (GP4DAT & 0xffefffff)

#define SET_SDO()		GP4DAT = (GP4DAT | 0x00200000)	//P4.5->DIN
#define CLR_SDO()		GP4DAT = (GP4DAT & 0xffdfffff)

void delay (int length)
{
	while (length >0)
    	length--;
}

//---------------------------------
//void WriteToAD7190(unsigned char count,unsigned char *buf);
//---------------------------------
//Function that writes to the AD7190 via the SPI port. 
//--------------------------------------------------------------------------------
void WriteToAD7190(unsigned char count, unsigned char *buf)
{
	unsigned	char	ValueToWrite = 0;
    unsigned	char	i = 0;
	unsigned	char	j = 0;
		
	SET_SCL();
	delay(1);
	SET_CS();
	delay(1);
	CLR_CS();
	delay(1);

	for(i=count;i>0;i--)
 	{
	 	ValueToWrite = *(buf + i - 1);
		for(j=0; j<8; j++)
		{
			CLR_SCL();
			if(0x80 == (ValueToWrite & 0x80))
			{
				SET_SDO();	  //Send one to SDO pin
			}
			else
			{
				CLR_SDO();	  //Send zero to SDO pin
			}
			delay(1);
			SET_SCL();
			delay(1);
			ValueToWrite <<= 1;	//Rotate data
		}
	}
	SET_CS();
}


//---------------------------------
//void ReadFromAD7190(unsigned char count,unsigned char *buf)
//---------------------------------
//Function that reads from the AD7190 via the SPI port. 
//--------------------------------------------------------------------------------
void ReadFromAD7190(unsigned char count, unsigned char *buf)
{
	unsigned	char	i = 0;
	unsigned	char	j = 0;
	unsigned	int  	iTemp = 0;
	unsigned	char  	RotateData = 0;

	SET_SCL();
	delay(1);
	SET_CS();
	delay(1);
	CLR_CS();
	delay(1);

	for(j=count; j>0; j--)
	{
		for(i=0; i<8; i++)
		{
		    CLR_SCL();
			RotateData <<= 1;		//Rotate data
			delay(1);
			iTemp = GP0DAT;			//Read SDI of AD7190
			SET_SCL();	
			if(0x00000020 == (iTemp & 0x00000020))
			{
				RotateData |= 1;	
			}
			delay(1);
		}
		*(buf + j - 1)= RotateData;
	}	 
	SET_CS();
} 


